"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateSamples = void 0;
const fs = require("fs");
const path = require("path");
const hbs = require("handlebars");
const autorestSession_1 = require("../../autorestSession");
/**
 * Function that writes the code for all the operations.
 * It will generate one file per operation group and each file contains:
 *    - A class definition for the operation group
 *    - Methods and overrides for each operation
 *    - OperationSpecs for each operation
 * @param clientDetails client details
 * @param project project for code generation
 */
function generateSamples(clientDetails, project) {
    // Toplevel operations are inlined in the client
    const samples = clientDetails.samples;
    const session = autorestSession_1.getSession();
    if (!samples) {
        session.error("No samples are found! ", []);
    }
    for (const sample of samples) {
        try {
            const file = fs.readFileSync(path.join(__dirname, "../static/samples.ts.hbs"), {
                encoding: "utf-8"
            });
            const readmeFileContents = hbs.compile(file, { noEscape: true });
            project.createSourceFile(`samples-dev/${sample.sampleFunctionName}.ts`, readmeFileContents(sample), {
                overwrite: true
            });
        }
        catch (error) {
            session.error("An error was encountered while handling sample generation", [sample.sampleFunctionName]);
            throw error;
        }
    }
}
exports.generateSamples = generateSamples;
//# sourceMappingURL=sampleGenerator.js.map