"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generatePathFirstClient = exports.pathDictionary = void 0;
const codemodel_1 = require("@autorest/codemodel");
const operationHelpers_1 = require("./operationHelpers");
const ts_morph_1 = require("ts-morph");
const path = require("path");
const autorestSession_1 = require("../autorestSession");
const urlTransforms_1 = require("../transforms/urlTransforms");
const nameUtils_1 = require("../utils/nameUtils");
const languageHelpers_1 = require("../utils/languageHelpers");
const operationHelpers_2 = require("./helpers/operationHelpers");
const generateMethodShortcuts_1 = require("./generateMethodShortcuts");
exports.pathDictionary = {};
function generatePathFirstClient(model, project) {
    var _a, _b, _c, _d, _e;
    const name = nameUtils_1.normalizeName(languageHelpers_1.getLanguageMetadata(model.language).name, nameUtils_1.NameType.File);
    const { srcPath } = autorestSession_1.getAutorestOptions();
    const clientFile = project.createSourceFile(path.join(srcPath, `clientDefinitions.ts`), undefined, {
        overwrite: true
    });
    // Get all paths
    const importedParameters = new Set();
    const importedResponses = new Set();
    exports.pathDictionary = {};
    for (const operationGroup of model.operationGroups) {
        for (const operation of operationGroup.operations) {
            const operationName = languageHelpers_1.getLanguageMetadata(operation.language).name;
            const operationDescription = languageHelpers_1.getLanguageMetadata(operation.language)
                .description;
            const pathParameters = ((_a = operation.parameters) === null || _a === void 0 ? void 0 : _a.filter(p => { var _a; return ((_a = p.protocol.http) === null || _a === void 0 ? void 0 : _a.in) === codemodel_1.ParameterLocation.Path; }).map(p => {
                const languageMetadata = languageHelpers_1.getLanguageMetadata(p.language);
                return {
                    name: languageMetadata.serializedName || languageMetadata.name,
                    schema: p.schema,
                    description: languageMetadata.description
                };
            })) || [];
            const path = (_c = (_b = operation.requests) === null || _b === void 0 ? void 0 : _b[0].protocol.http) === null || _c === void 0 ? void 0 : _c.path;
            pathParameters.sort(function compare(a, b) {
                return path.indexOf(a.name) - path.indexOf(b.name);
            });
            for (const request of operation.requests || []) {
                const path = ((_d = request.protocol.http) === null || _d === void 0 ? void 0 : _d.path) || "";
                const method = (_e = request.protocol.http) === null || _e === void 0 ? void 0 : _e.method;
                if (path && method) {
                    if (!exports.pathDictionary[path]) {
                        exports.pathDictionary[path] = {
                            pathParameters,
                            methods: {},
                            name: operationName
                        };
                    }
                    const hasOptionalOptions = !hasRequiredOptions(operation);
                    const newMethod = {
                        description: operationDescription,
                        optionsName: getOperationOptionsType(operation, importedParameters),
                        hasOptionalOptions,
                        returnType: `Promise<${getOperationReturnType(operation, importedResponses)}>`
                    };
                    if (exports.pathDictionary[path].methods[`${method}`]) {
                        exports.pathDictionary[path].methods[`${method}`].push(newMethod);
                    }
                    else {
                        exports.pathDictionary[path].methods[`${method}`] = [newMethod];
                    }
                }
            }
        }
    }
    writeShortcutInterface(model, exports.pathDictionary, clientFile);
    clientFile.addInterface({
        name: "Routes",
        isExported: true,
        callSignatures: getPathFirstRoutesInterfaceDefinition(exports.pathDictionary, clientFile)
    });
    const clientName = languageHelpers_1.getLanguageMetadata(model.language).name;
    const uriParameter = getClientUriParameter();
    const { addCredentials, credentialKeyHeaderName } = autorestSession_1.getAutorestOptions();
    const credentialTypes = addCredentials ? ["TokenCredential"] : [];
    if (credentialKeyHeaderName) {
        credentialTypes.push("KeyCredential");
    }
    const clientIterfaceName = `${clientName}RestClient`;
    const { rlcShortcut } = autorestSession_1.getAutorestOptions();
    let shortcutElements = !rlcShortcut
        ? []
        : model.operationGroups.map(og => {
            const groupName = og.language.default.name;
            const name = nameUtils_1.normalizeName(groupName, nameUtils_1.NameType.OperationGroup, true, generateMethodShortcuts_1.REST_CLIENT_RESERVED, nameUtils_1.CasingConvention.Camel);
            const interfaceName = nameUtils_1.normalizeName(`${name}Operations`, nameUtils_1.NameType.Interface, true, generateMethodShortcuts_1.REST_CLIENT_RESERVED);
            return { name, type: interfaceName };
        });
    // There may be operations without an operation group, those shortcut
    // methods need to be handled differently.
    const shortcutsInOperationGroup = shortcutElements.filter(s => s.name);
    clientFile.addTypeAlias({
        isExported: true,
        name: clientIterfaceName,
        type: ts_morph_1.Writers.intersectionType("Client", ts_morph_1.Writers.objectType({
            properties: [
                { name: "path", type: "Routes" },
                ...shortcutsInOperationGroup
            ]
        }), 
        // If the length of shortcutMethods in operation group and all shortcutMethods
        // is the same, then we don't have any operations at the client level
        // Otherwise we need to make the client interface name an union with the
        // definition of all client level shortcut methods
        ...(shortcutsInOperationGroup.length !== shortcutElements.length
            ? [`ClientOperations`]
            : []))
    });
    if (importedParameters.size) {
        clientFile.addImportDeclaration({
            namedImports: [...importedParameters],
            moduleSpecifier: "./parameters"
        });
    }
    if (importedResponses.size) {
        clientFile.addImportDeclaration({
            namedImports: [...importedResponses],
            moduleSpecifier: "./responses"
        });
    }
    clientFile.addImportDeclarations([
        {
            namedImports: ["Client"],
            moduleSpecifier: "@azure-rest/core-client"
        }
    ]);
    clientFile.addImportDeclarations([
        {
            namedImports: credentialTypes,
            moduleSpecifier: "@azure/core-auth"
        }
    ]);
}
exports.generatePathFirstClient = generatePathFirstClient;
function writeShortcutInterface(model, pathDictionary, clientFile) {
    const { rlcShortcut } = autorestSession_1.getAutorestOptions();
    if (!rlcShortcut) {
        return;
    }
    // Create a map of Operation group descriptions
    const descriptions = model.operationGroups.reduce((map, current) => {
        const { name, description } = current.language.default;
        map.set(name, description);
        return map;
    }, new Map());
    const shortcuts = generateMethodShortcuts_1.generateMethodShortcuts(model, pathDictionary);
    for (const group of Object.keys(shortcuts)) {
        const groupName = nameUtils_1.normalizeName(group, nameUtils_1.NameType.Interface, true, generateMethodShortcuts_1.REST_CLIENT_RESERVED) ||
            "Client";
        const groupOperations = shortcuts[group];
        clientFile.addInterface({
            docs: [
                descriptions.get(group) ||
                    `Contains operations for ${groupName} operations`
            ],
            name: `${groupName}Operations`,
            isExported: true,
            methods: groupOperations
        });
    }
}
function hasRequiredOptions(operation) {
    return operationHelpers_2.getOperationParameters(operation)
        .filter(p => p.implementation === codemodel_1.ImplementationLocation.Method)
        .filter(p => { var _a; return ["query", "body", "headers"].includes((_a = p.protocol.http) === null || _a === void 0 ? void 0 : _a.in); })
        .some(p => p.required);
}
function getOperationOptionsType(operation, importedParameters = new Set()) {
    const paramsName = `${languageHelpers_1.getLanguageMetadata(operation.language).name}Parameters`;
    importedParameters.add(paramsName);
    return paramsName;
}
function getOperationReturnType(operation, importedResponses = new Set()) {
    let returnType = "HttpResponse";
    if (operation.responses && operation.responses.length) {
        const responses = [...operation.responses, ...(operation.exceptions || [])];
        const responseTypes = responses
            .filter(r => { var _a, _b; return ((_a = r.protocol.http) === null || _a === void 0 ? void 0 : _a.statusCodes) && ((_b = r.protocol.http) === null || _b === void 0 ? void 0 : _b.statusCodes.length); })
            .map(r => {
            const responseName = operationHelpers_1.getResponseTypeName(operation, r);
            importedResponses.add(responseName);
            return responseName;
        });
        if (responseTypes.length) {
            returnType = responseTypes.join(" | ");
        }
    }
    return returnType;
}
function getPathFirstRoutesInterfaceDefinition(paths, sourcefile) {
    const signatures = [];
    for (const key of Object.keys(paths)) {
        generatePathFirstRouteMethodsDefinition(paths[key].name, paths[key].methods, sourcefile);
        const pathParams = paths[key].pathParameters;
        signatures.push({
            docs: [
                `Resource for '${key
                    .replace(/}/g, "\\}")
                    .replace(/{/g, "\\{")}' has methods for the following verbs: ${Object.keys(paths[key].methods).join(", ")}`
            ],
            parameters: [
                { name: "path", type: `"${key}"` },
                ...operationHelpers_2.getPathParamDefinitions(pathParams)
            ],
            returnType: paths[key].name,
            kind: ts_morph_1.StructureKind.CallSignature
        });
    }
    return signatures;
}
function getClientUriParameter() {
    const { model } = autorestSession_1.getSession();
    const { parameterName } = urlTransforms_1.transformBaseUrl(model);
    return parameterName;
}
function generatePathFirstRouteMethodsDefinition(operationName, methods, file) {
    const methodDefinitions = operationHelpers_2.buildMethodDefinitions(methods);
    file.addInterface({
        methods: methodDefinitions,
        name: operationName,
        isExported: true
    });
}
//# sourceMappingURL=generateClientDefinition.js.map