"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateMethodShortcutImplementation = exports.generateMethodShortcuts = exports.REST_CLIENT_RESERVED = void 0;
const nameUtils_1 = require("../utils/nameUtils");
const operationHelpers_1 = require("./helpers/operationHelpers");
exports.REST_CLIENT_RESERVED = [
    { name: "path", reservedFor: [nameUtils_1.NameType.Property, nameUtils_1.NameType.OperationGroup] },
    {
        name: "pathUnchecked",
        reservedFor: [nameUtils_1.NameType.Property, nameUtils_1.NameType.OperationGroup]
    },
    {
        name: "pipeline",
        reservedFor: [nameUtils_1.NameType.Property, nameUtils_1.NameType.OperationGroup]
    }
];
function generateMethodShortcuts(model, paths) {
    const operationGroups = model.operationGroups;
    let keys = {};
    for (const group of operationGroups) {
        const groupName = nameUtils_1.normalizeName(group.language.default.name, nameUtils_1.NameType.OperationGroup, true, exports.REST_CLIENT_RESERVED, nameUtils_1.CasingConvention.Camel);
        keys[groupName] = buildOperationDefinitions(group.operations, paths);
    }
    return keys;
}
exports.generateMethodShortcuts = generateMethodShortcuts;
function generateMethodShortcutImplementation(model, paths) {
    const operationGroups = model.operationGroups;
    let keys = {};
    for (const group of operationGroups) {
        const groupName = nameUtils_1.normalizeName(group.language.default.name, nameUtils_1.NameType.OperationGroup, true, exports.REST_CLIENT_RESERVED, nameUtils_1.CasingConvention.Camel);
        keys[groupName] = buildOperationDeclarations(group.operations, paths);
    }
    return keys;
}
exports.generateMethodShortcutImplementation = generateMethodShortcutImplementation;
function buildOperationDeclarations(operations, paths) {
    var _a, _b, _c;
    let ops = [];
    for (const operation of operations) {
        const name = nameUtils_1.normalizeName(operation.language.default.name, nameUtils_1.NameType.Property, true);
        const path = (_a = operation.requests[0].protocol.http) === null || _a === void 0 ? void 0 : _a.path;
        const method = (_b = operation.requests[0].protocol.http) === null || _b === void 0 ? void 0 : _b.method;
        const pathParams = (_c = paths[path]) === null || _c === void 0 ? void 0 : _c.pathParameters;
        const methodDefinitions = generateOperationDeclaration(path, name, method, pathParams);
        ops = [...ops, methodDefinitions];
    }
    return ops;
}
function generateOperationDeclaration(path, operationName, method, pathParams = []) {
    const pathParamNames = `${pathParams.length > 0 ? `${pathParams.map(p => p.name)},` : ""}`;
    return `"${operationName}": (${pathParamNames} options) => {
    return client.path("${path}", ${pathParamNames}).${method}(options);
  }`;
}
function buildOperationDefinitions(operations, paths) {
    var _a, _b, _c, _d;
    let ops = [];
    for (const operation of operations) {
        const name = nameUtils_1.normalizeName(operation.language.default.name, nameUtils_1.NameType.Property, true);
        const path = (_a = operation.requests[0].protocol.http) === null || _a === void 0 ? void 0 : _a.path;
        const method = (_b = operation.requests[0].protocol.http) === null || _b === void 0 ? void 0 : _b.method;
        const methods = (_c = paths[path]) === null || _c === void 0 ? void 0 : _c.methods[method];
        const pathParams = (_d = paths[path]) === null || _d === void 0 ? void 0 : _d.pathParameters;
        const methodDefinitions = operationHelpers_1.buildMethodDefinitions({ [name]: methods }, pathParams);
        ops = [...ops, ...methodDefinitions];
    }
    return ops;
}
//# sourceMappingURL=generateMethodShortcuts.js.map