"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generatePagingHelper = void 0;
const fs_1 = require("fs");
const autorestSession_1 = require("../autorestSession");
const extractPaginationDetails_1 = require("../utils/extractPaginationDetails");
const path = require("path");
const hbs = require("handlebars");
function generatePagingHelper(project) {
    const pagingInfo = getPagingInfo();
    let file = "";
    file = fs_1.readFileSync(path.join(__dirname, "paginateHelper.ts.hbs"), {
        encoding: "utf-8"
    });
    hbs.registerHelper("quoteWrap", function (value) {
        if (Array.isArray(value)) {
            return value.map(element => `"${element}"`).join();
        }
        return `"${value}"`;
    });
    const { srcPath } = autorestSession_1.getAutorestOptions();
    const readmeFileContents = hbs.compile(file, { noEscape: true });
    project.createSourceFile(path.join(srcPath, "paginateHelper.ts"), readmeFileContents(pagingInfo), {
        overwrite: true
    });
}
exports.generatePagingHelper = generatePagingHelper;
function getPagingInfo() {
    const { model } = autorestSession_1.getSession();
    const nextLinks = new Set();
    const itemNames = new Set();
    // Add default values
    nextLinks.add("nextLink");
    itemNames.add("value");
    for (let operationGroup of model.operationGroups) {
        for (let operation of operationGroup.operations) {
            const paginationDetails = extractPaginationDetails_1.extractPaginationDetails(operation);
            if (paginationDetails) {
                const { nextLinkName, itemName } = paginationDetails;
                nextLinkName && nextLinks.add(`${nextLinkName}`);
                itemName && itemNames.add(`${itemName}`);
            }
        }
    }
    // If there are more than one options for nextLink and item names we need to generate a
    // more complex pagination helper.
    const isComplexPaging = nextLinks.size > 1 || itemNames.size > 1;
    return {
        itemNames: [...itemNames],
        nextLinkNames: [...nextLinks],
        isComplexPaging
    };
}
//# sourceMappingURL=generatePagingHelper.js.map