"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateSchemaTypes = void 0;
const codemodel_1 = require("@autorest/codemodel");
const path = require("path");
const generateObjectTypes_1 = require("./generateObjectTypes");
const autorestSession_1 = require("../autorestSession");
/**
 * Generates types to represent schema definitions in the swagger
 */
function generateSchemaTypes(model, project) {
    // Track models that need to be imported
    const importedModels = new Set();
    const objectsDefinitions = generateObjectTypes_1.buildObjectInterfaces(model, importedModels, [
        codemodel_1.SchemaContext.Input
    ]);
    const objectTypeAliases = generateObjectTypes_1.buildPolymorphicAliases(model, [
        codemodel_1.SchemaContext.Input
    ]);
    const { srcPath } = autorestSession_1.getAutorestOptions();
    if (objectTypeAliases.length || objectsDefinitions.length) {
        const inputModelsFile = project.createSourceFile(path.join(srcPath, `models.ts`), undefined, {
            overwrite: true
        });
        inputModelsFile.addInterfaces(objectsDefinitions);
        inputModelsFile.addTypeAliases(objectTypeAliases);
    }
    const outputObjectsDefinitions = generateObjectTypes_1.buildObjectInterfaces(model, importedModels, [codemodel_1.SchemaContext.Output, codemodel_1.SchemaContext.Exception]);
    const outputObjectTypeAliases = generateObjectTypes_1.buildPolymorphicAliases(model, [
        codemodel_1.SchemaContext.Output,
        codemodel_1.SchemaContext.Exception
    ]);
    if (outputObjectTypeAliases.length || outputObjectsDefinitions.length) {
        const outputModelsFile = project.createSourceFile(path.join(srcPath, `outputModels.ts`), undefined, {
            overwrite: true
        });
        outputModelsFile.addInterfaces(outputObjectsDefinitions);
        outputModelsFile.addTypeAliases(outputObjectTypeAliases);
    }
}
exports.generateSchemaTypes = generateSchemaTypes;
//# sourceMappingURL=generateSchemaTypes.js.map