"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateTopLevelIndexFile = void 0;
const path = require("path");
const autorestSession_1 = require("../autorestSession");
const nameUtils_1 = require("../utils/nameUtils");
const batchOutputFolder = [];
function generateTopLevelIndexFile(model, project) {
    const { batch, srcPath } = autorestSession_1.getAutorestOptions();
    if (srcPath) {
        const clientName = model.language.default.name;
        const moduleName = nameUtils_1.normalizeName(clientName, nameUtils_1.NameType.File);
        const relativePath = srcPath.replace('/src', '');
        batchOutputFolder.push([relativePath, clientName, moduleName]);
    }
    if (batch && batch.length > 1 && batchOutputFolder.length === batch.length) {
        const { srcPath } = autorestSession_1.getAutorestOptions();
        const fileDirectory = path.join(srcPath, '../../');
        const file = project.createSourceFile('/src/index.ts', undefined, {
            overwrite: true
        });
        file.moveToDirectory(fileDirectory);
        const allModules = [];
        batchOutputFolder.forEach(item => {
            file.addImportDeclaration({
                namespaceImport: item[1],
                moduleSpecifier: `${item[0]}`
            });
            file.addExportDeclaration({
                moduleSpecifier: `${item[0]}/${item[2]}`,
                namedExports: [`${item[1]} as ${item[1]}Client`]
            });
            allModules.push(item[1]);
        });
        file.addExportDeclaration({
            namedExports: [...allModules]
        });
    }
}
exports.generateTopLevelIndexFile = generateTopLevelIndexFile;
//# sourceMappingURL=generateTopLevelIndexFile.js.map