"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDocs = exports.getPropertySignature = void 0;
const ts_morph_1 = require("ts-morph");
const languageHelpers_1 = require("../utils/languageHelpers");
const schemaHelpers_1 = require("./schemaHelpers");
/**
 * Builds a Typescript property or parameter signature
 * @param property - Property or parameter to get the Typescript signature for
 * @param importedModels - Set to track the models that need to be imported
 * @returns a PropertySignatureStructure for the property.
 */
function getPropertySignature(property, schemaUsage, importedModels = new Set()) {
    var _a;
    const propertyLangMetadata = languageHelpers_1.getLanguageMetadata(property.language);
    const propertyName = `"${(_a = propertyLangMetadata.serializedName) !== null && _a !== void 0 ? _a : property.serializedName}"`;
    if (!propertyName) {
        throw new Error(`Couldn't find name for property ${JSON.stringify(propertyLangMetadata)}`);
    }
    const description = getDocs(property);
    const type = schemaHelpers_1.getElementType(property.schema, schemaUsage, importedModels);
    return Object.assign(Object.assign({ name: propertyName }, (description && { docs: [{ description }] })), { hasQuestionToken: !property.required, type, kind: ts_morph_1.StructureKind.PropertySignature });
}
exports.getPropertySignature = getPropertySignature;
function getDocs(property) {
    const { description } = languageHelpers_1.getLanguageMetadata(property.language);
    const docs = [];
    if (description) {
        docs.push(description);
    }
    const formatDocs = schemaHelpers_1.getFormatDocs(property.schema);
    if (formatDocs) {
        docs.push(formatDocs);
    }
    return docs.join("\n\n");
}
exports.getDocs = getDocs;
//# sourceMappingURL=getPropertySignature.js.map