"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPathParamDefinitions = exports.buildMethodDefinitions = exports.getOperationParameters = void 0;
const codemodel_1 = require("@autorest/codemodel");
const schemaHelpers_1 = require("../schemaHelpers");
/**
 * Given an operation, extract all its parameters
 */
function getOperationParameters(operation, requestIndex = 0) {
    var _a, _b, _c, _d;
    const operationParams = [
        ...((_a = operation.parameters) !== null && _a !== void 0 ? _a : []),
        ...((_b = operation.signatureParameters) !== null && _b !== void 0 ? _b : [])
    ];
    const distinctParams = new Set(operationParams);
    if (operation.requests) {
        [
            ...((_c = operation.requests[requestIndex].parameters) !== null && _c !== void 0 ? _c : []),
            ...((_d = operation.requests[requestIndex].signatureParameters) !== null && _d !== void 0 ? _d : [])
        ].forEach(p => distinctParams.add(p));
    }
    return [...distinctParams].filter(filterMethodNotSynthetic);
}
exports.getOperationParameters = getOperationParameters;
function filterMethodNotSynthetic(parameter) {
    const isApiVersion = parameter.origin &&
        parameter.language.default.serializedName === "api-version" &&
        parameter.implementation === codemodel_1.ImplementationLocation.Method;
    // Origin is added by M4 on synthetic parameters
    return ((!parameter.origin || isApiVersion) &&
        parameter.implementation === codemodel_1.ImplementationLocation.Method);
}
function buildMethodDefinitions(methods, pathParams = []) {
    const methodDefinitions = [];
    for (const key of Object.keys(methods)) {
        const method = methods[key];
        const description = methods[key][0].description;
        let areAllOptional = methods[key][0].hasOptionalOptions;
        methodDefinitions.push(Object.assign(Object.assign({ name: key }, (description && { docs: [{ description }] })), { parameters: [
                ...getPathParamDefinitions(pathParams),
                {
                    name: "options",
                    hasQuestionToken: areAllOptional,
                    type: method.map(m => m.optionsName).join(" | ")
                }
            ], returnType: method.map(m => m.returnType).join(" | ") }));
    }
    return methodDefinitions;
}
exports.buildMethodDefinitions = buildMethodDefinitions;
function getPathParamDefinitions(pathParams) {
    return pathParams.map(p => {
        return {
            name: p.name,
            type: schemaHelpers_1.getElementType(p.schema, [
                codemodel_1.SchemaContext.Input,
                codemodel_1.SchemaContext.Exception
            ]),
            description: p.description
        };
    });
}
exports.getPathParamDefinitions = getPathParamDefinitions;
//# sourceMappingURL=operationHelpers.js.map