"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllExamples = exports.transformSamples = void 0;
const codemodel_1 = require("@autorest/codemodel");
const languageHelpers_1 = require("../utils/languageHelpers");
const autorestSession_1 = require("../autorestSession");
const nameUtils_1 = require("../utils/nameUtils");
const operationsUtils_1 = require("../generators/utils/operationsUtils");
const codegen_1 = require("@azure-tools/codegen");
const pagingOperations_1 = require("../generators/utils/pagingOperations");
function transformSamples(codeModel, clientDetails) {
    return __awaiter(this, void 0, void 0, function* () {
        return yield getAllExamples(codeModel, clientDetails);
    });
}
exports.transformSamples = transformSamples;
function getTransformedOperationGroup(operationGroup, operationGroupDetails) {
    return operationGroupDetails.filter(operationGroupDetail => operationGroup.$key === operationGroupDetail.originalKey)[0];
}
function getTransformedOperation(operationGroup, operation, operationGroupDetails, clientName) {
    return operationGroupDetails.operations.filter(operationDetail => operationDetail.fullName === nameUtils_1.getOperationFullName(operationGroup, operation, clientName))[0];
}
function getAllExamples(codeModel, clientDetails) {
    var _a, _b, _c, _d;
    return __awaiter(this, void 0, void 0, function* () {
        const operationGroupDetails = clientDetails.operationGroups;
        const { packageDetails } = autorestSession_1.getAutorestOptions();
        const session = autorestSession_1.getSession();
        let examplesModels = [];
        if (((_b = (_a = codeModel === null || codeModel === void 0 ? void 0 : codeModel.testModel) === null || _a === void 0 ? void 0 : _a.mockTest) === null || _b === void 0 ? void 0 : _b.exampleGroups) !== undefined) {
            for (const exampleGroups of codeModel.testModel.mockTest.exampleGroups) {
                try {
                    for (const example of exampleGroups.examples) {
                        const clientName = languageHelpers_1.getLanguageMetadata(codeModel.language).name;
                        const ogDetails = getTransformedOperationGroup(example.operationGroup, operationGroupDetails);
                        if (ogDetails === undefined) {
                            session.error("An error was encountered while transforming sample", [
                                exampleGroups.operationId
                            ]);
                            continue;
                        }
                        const opDetails = getTransformedOperation(example.operationGroup, example.operation, ogDetails, clientName);
                        let methodName = operationsUtils_1.calculateMethodName(opDetails);
                        if (opDetails.isLro && opDetails.pagination === undefined) {
                            methodName = `${methodName}AndWait`;
                        }
                        else if (opDetails.pagination) {
                            methodName = pagingOperations_1.getPublicMethodName(opDetails);
                        }
                        const opGroupName = ogDetails.name;
                        const sample = {
                            operationDescription: languageHelpers_1.getLanguageMetadata(example.operation.language).description,
                            operationName: methodName,
                            operationGroupName: nameUtils_1.normalizeName(opGroupName, nameUtils_1.NameType.Property),
                            clientClassName: clientName,
                            clientPackageName: packageDetails.name,
                            clientParameterNames: "",
                            methodParameterNames: "",
                            bodySchemaName: "",
                            hasBody: false,
                            hasOptional: false,
                            sampleFunctionName: codegen_1.camelCase(example.name.replace(/\//g, " Or ").replace(/,|\.|\(|\)/g, " ").replace('\'s ', ' ')),
                            methodParamAssignments: [],
                            clientParamAssignments: [],
                            isTopLevel: ogDetails.isTopLevel,
                            isPaging: opDetails.pagination !== undefined,
                            originalFileLocation: example.originalFile
                        };
                        const clientParameterNames = ["credential"];
                        const requiredParams = clientDetails.parameters.filter(param => param.required &&
                            param.implementationLocation === codemodel_1.ImplementationLocation.Client &&
                            !param.defaultValue &&
                            param.schemaType !== codemodel_1.SchemaType.Constant);
                        for (const clientParameter of example.clientParameters) {
                            if (clientParameter.exampleValue.schema.type === codemodel_1.SchemaType.Constant) {
                                continue;
                            }
                            const parameterName = nameUtils_1.normalizeName(languageHelpers_1.getLanguageMetadata(clientParameter.exampleValue.language).name, nameUtils_1.NameType.Parameter);
                            const paramAssignment = `const ${parameterName} = ` +
                                getParameterAssignment(clientParameter.exampleValue);
                            sample.clientParamAssignments.push(paramAssignment);
                            clientParameterNames.push(parameterName);
                        }
                        if (clientParameterNames.indexOf("subscriptionId") < 0 && requiredParams.find(param => param.name === 'subscriptionId')) {
                            const subscriptionIdAssignment = `const subscriptionId = "00000000-0000-0000-0000-000000000000"`;
                            sample.clientParamAssignments.push(subscriptionIdAssignment);
                            clientParameterNames.push('subscriptionId');
                        }
                        if (clientParameterNames.length > 0) {
                            sample.clientParameterNames = clientParameterNames.join(", ");
                        }
                        const methodParameterNames = [];
                        const optionalParams = [];
                        for (const methodParameter of example.methodParameters) {
                            if (methodParameter.exampleValue.schema.type === codemodel_1.SchemaType.Constant) {
                                continue;
                            }
                            const parameterName = languageHelpers_1.getLanguageMetadata(methodParameter.exampleValue.language).name;
                            let paramAssignment = "";
                            if (((_d = (_c = methodParameter.parameter.protocol) === null || _c === void 0 ? void 0 : _c.http) === null || _d === void 0 ? void 0 : _d["in"]) === "body") {
                                sample.hasBody = true;
                                sample.bodySchemaName = languageHelpers_1.getLanguageMetadata(methodParameter.exampleValue.schema.language).name;
                                paramAssignment =
                                    `const ${parameterName}: ${sample.bodySchemaName} = ` +
                                        getParameterAssignment(methodParameter.exampleValue);
                            }
                            else {
                                paramAssignment =
                                    `const ${parameterName} = ` +
                                        getParameterAssignment(methodParameter.exampleValue);
                            }
                            if (!methodParameter.parameter.required) {
                                optionalParams.push(parameterName);
                            }
                            else {
                                methodParameterNames.push(parameterName);
                            }
                            sample.methodParamAssignments.push(paramAssignment);
                        }
                        if (optionalParams.length > 0) {
                            const optionAssignment = `const options = {${optionalParams
                                .map(item => {
                                return item + ": " + item;
                            })
                                .join(", ")}}`;
                            sample.methodParamAssignments.push(optionAssignment);
                            methodParameterNames.push("options");
                        }
                        if (methodParameterNames.length > 0) {
                            sample.methodParameterNames = methodParameterNames.join(", ");
                        }
                        examplesModels.push(sample);
                    }
                }
                catch (error) {
                    session.error("An error was encountered while transforming sample", [
                        exampleGroups.operationId
                    ]);
                    throw error;
                }
            }
        }
        return examplesModels;
    });
}
exports.getAllExamples = getAllExamples;
function getParameterAssignment(exampleValue) {
    var _a, _b, _c, _d;
    let schemaType = exampleValue.schema.type;
    const rawValue = exampleValue.rawValue;
    let retValue = rawValue;
    switch (schemaType) {
        case codemodel_1.SchemaType.Constant:
            const contentSchema = exampleValue.schema;
            schemaType = contentSchema.valueType.type;
            break;
        case codemodel_1.SchemaType.Choice:
        case codemodel_1.SchemaType.SealedChoice:
            const choiceSchema = exampleValue.schema;
            schemaType = choiceSchema.choiceType.type;
            break;
    }
    if (rawValue === null) {
        switch (schemaType) {
            case codemodel_1.SchemaType.Object:
            case codemodel_1.SchemaType.Any:
            case codemodel_1.SchemaType.Dictionary:
                retValue = `{}`;
                break;
            case codemodel_1.SchemaType.Array:
                retValue = `[]`;
                break;
            default:
                retValue = undefined;
        }
        return retValue;
    }
    switch (schemaType) {
        case codemodel_1.SchemaType.String:
        case codemodel_1.SchemaType.Char:
        case codemodel_1.SchemaType.Time:
        case codemodel_1.SchemaType.Uuid:
        case codemodel_1.SchemaType.Uri:
        case codemodel_1.SchemaType.Credential:
        case codemodel_1.SchemaType.Duration:
            retValue = `"${rawValue === null || rawValue === void 0 ? void 0 : rawValue.toString().replace(/"/g, '\\"').replace(/\n/g, '\\n')}"`;
            break;
        case codemodel_1.SchemaType.Boolean:
            (retValue = rawValue), toString();
            break;
        case codemodel_1.SchemaType.Object:
        case codemodel_1.SchemaType.Dictionary:
            const values = [];
            for (const prop in exampleValue.properties) {
                const property = exampleValue.properties[prop];
                if (property === undefined || property === null) {
                    continue;
                }
                const initPropName = ((_b = (_a = property.language) === null || _a === void 0 ? void 0 : _a.default) === null || _b === void 0 ? void 0 : _b.name)
                    ? (_d = (_c = property.language) === null || _c === void 0 ? void 0 : _c.default) === null || _d === void 0 ? void 0 : _d.name
                    : prop;
                const propName = nameUtils_1.normalizeName(initPropName, nameUtils_1.NameType.Property, true);
                let propRetValue;
                if (propName.indexOf("/") > -1 || propName.match(/^\d/)) {
                    propRetValue = `"${propName}": ` + getParameterAssignment(property);
                }
                else {
                    propRetValue = `${propName}: ` + getParameterAssignment(property);
                }
                values.push(propRetValue);
            }
            if (values.length > 0) {
                retValue = `{${values.join(", ")}}`;
            }
            else {
                retValue = "{}";
            }
            break;
        case codemodel_1.SchemaType.Array:
            const valuesArr = [];
            for (const element of exampleValue.elements) {
                let propRetValueArr = getParameterAssignment(element);
                valuesArr.push(propRetValueArr);
            }
            if (valuesArr.length > 0) {
                retValue = `[${valuesArr.join(", ")}]`;
            }
            else {
                retValue = "[]";
            }
            break;
        case codemodel_1.SchemaType.Date:
        case codemodel_1.SchemaType.DateTime:
            retValue = `new Date("${rawValue}")`;
            break;
        case codemodel_1.SchemaType.Any:
        case codemodel_1.SchemaType.AnyObject:
            retValue = `${JSON.stringify(rawValue)}`;
            break;
        default:
            break;
    }
    return retValue;
}
//# sourceMappingURL=samplesTransforms.js.map