"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.headersToSchema = void 0;
const codemodel_1 = require("@autorest/codemodel");
const languageHelpers_1 = require("../utils/languageHelpers");
function headersToSchema(headers, operationFullName, isError) {
    if (!headers || !headers.length) {
        return undefined;
    }
    const headersSchema = new codemodel_1.ObjectSchema(`${operationFullName}${isError ? "Exception" : ""}Headers`, `Defines headers for ${operationFullName} operation.`);
    headers.forEach(({ header, language, schema, extensions }) => {
        if (!headersSchema.properties) {
            headersSchema.properties = [];
        }
        const { description, name } = languageHelpers_1.getLanguageMetadata(language);
        const prop = new codemodel_1.Property(name, description, schema, {
            // core-http serializer requires Header serialized names to be lowercase
            serializedName: header.toLocaleLowerCase()
        });
        if (extensions && extensions["x-ms-header-collection-prefix"]) {
            prop.schema.language.default["headerCollectionPrefix"] =
                extensions["x-ms-header-collection-prefix"];
        }
        headersSchema.properties.push(prop);
    });
    return headersSchema;
}
exports.headersToSchema = headersToSchema;
//# sourceMappingURL=headersToSchema.js.map